"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var http2 = require("http2");
var stream_1 = require("stream");
var constants_1 = require("./constants");
var metadata_1 = require("./metadata");
var _a = http2.constants, HTTP2_HEADER_STATUS = _a.HTTP2_HEADER_STATUS, HTTP2_HEADER_CONTENT_TYPE = _a.HTTP2_HEADER_CONTENT_TYPE;
var ReadState;
(function (ReadState) {
    ReadState[ReadState["NO_DATA"] = 0] = "NO_DATA";
    ReadState[ReadState["READING_SIZE"] = 1] = "READING_SIZE";
    ReadState[ReadState["READING_MESSAGE"] = 2] = "READING_MESSAGE";
})(ReadState || (ReadState = {}));
var emptyBuffer = Buffer.alloc(0);
var Http2CallStream = /** @class */ (function (_super) {
    __extends(Http2CallStream, _super);
    function Http2CallStream(methodName, options, filterStackFactory) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.methodName = methodName;
        _this.options = options;
        _this.statusEmitted = false;
        _this.http2Stream = null;
        _this.pendingRead = false;
        _this.pendingWrite = null;
        _this.pendingWriteCallback = null;
        _this.pendingFinalCallback = null;
        _this.readState = ReadState.NO_DATA;
        _this.readCompressFlag = false;
        _this.readPartialSize = Buffer.alloc(4);
        _this.readSizeRemaining = 4;
        _this.readMessageSize = 0;
        _this.readPartialMessage = [];
        _this.readMessageRemaining = 0;
        _this.unpushedReadMessages = [];
        // Status code mapped from :status. To be used if grpc-status is not received
        _this.mappedStatusCode = constants_1.Status.UNKNOWN;
        // This is populated (non-null) if and only if the call has ended
        _this.finalStatus = null;
        _this.filterStack = filterStackFactory.createFilter(_this);
        return _this;
    }
    Http2CallStream.prototype.endCall = function (status) {
        if (this.finalStatus === null) {
            this.finalStatus = status;
            this.emit('status', status);
        }
    };
    Http2CallStream.prototype.tryPush = function (messageBytes, canPush) {
        if (canPush) {
            if (!this.push(messageBytes)) {
                canPush = false;
                this.http2Stream.pause();
            }
        }
        else {
            this.unpushedReadMessages.push(messageBytes);
        }
        return canPush;
    };
    Http2CallStream.prototype.attachHttp2Stream = function (stream) {
        var _this = this;
        if (this.finalStatus !== null) {
            stream.rstWithCancel();
        }
        else {
            this.http2Stream = stream;
            stream.on('response', function (headers) {
                switch (headers[HTTP2_HEADER_STATUS]) {
                    // TODO(murgatroid99): handle 100 and 101
                    case '400':
                        _this.mappedStatusCode = constants_1.Status.INTERNAL;
                        break;
                    case '401':
                        _this.mappedStatusCode = constants_1.Status.UNAUTHENTICATED;
                        break;
                    case '403':
                        _this.mappedStatusCode = constants_1.Status.PERMISSION_DENIED;
                        break;
                    case '404':
                        _this.mappedStatusCode = constants_1.Status.UNIMPLEMENTED;
                        break;
                    case '429':
                    case '502':
                    case '503':
                    case '504':
                        _this.mappedStatusCode = constants_1.Status.UNAVAILABLE;
                        break;
                    default:
                        _this.mappedStatusCode = constants_1.Status.UNKNOWN;
                }
                delete headers[HTTP2_HEADER_STATUS];
                delete headers[HTTP2_HEADER_CONTENT_TYPE];
                var metadata;
                try {
                    metadata = metadata_1.Metadata.fromHttp2Headers(headers);
                }
                catch (e) {
                    _this.cancelWithStatus(constants_1.Status.UNKNOWN, e.message);
                    return;
                }
                _this.filterStack.receiveMetadata(Promise.resolve(metadata))
                    .then(function (finalMetadata) {
                    _this.emit('metadata', finalMetadata);
                }, function (error) {
                    _this.cancelWithStatus(constants_1.Status.UNKNOWN, error.message);
                });
            });
            stream.on('trailers', function (headers) {
                var code = _this.mappedStatusCode;
                if (headers.hasOwnProperty('grpc-status')) {
                    var receivedCode = Number(headers['grpc-status']);
                    if (receivedCode in constants_1.Status) {
                        code = receivedCode;
                    }
                    else {
                        code = constants_1.Status.UNKNOWN;
                    }
                    delete headers['grpc-status'];
                }
                var details = '';
                if (headers.hasOwnProperty('grpc-message')) {
                    details = decodeURI(headers['grpc-message']);
                }
                var metadata;
                try {
                    metadata = metadata_1.Metadata.fromHttp2Headers(headers);
                }
                catch (e) {
                    metadata = new metadata_1.Metadata();
                }
                var status = { code: code, details: details, metadata: metadata };
                _this.filterStack.receiveTrailers(Promise.resolve(status))
                    .then(function (finalStatus) {
                    _this.endCall(finalStatus);
                }, function (error) {
                    _this.endCall({
                        code: constants_1.Status.INTERNAL,
                        details: 'Failed to process received status',
                        metadata: new metadata_1.Metadata()
                    });
                });
            });
            stream.on('data', function (data) {
                var readHead = 0;
                var canPush = true;
                var toRead;
                while (readHead < data.length) {
                    switch (_this.readState) {
                        case ReadState.NO_DATA:
                            _this.readCompressFlag = (data.readUInt8(readHead) !== 0);
                            readHead += 1;
                            _this.readState = ReadState.READING_SIZE;
                            _this.readPartialSize.fill(0);
                            _this.readSizeRemaining = 4;
                            _this.readMessageSize = 0;
                            _this.readMessageRemaining = 0;
                            _this.readPartialMessage = [];
                            break;
                        case ReadState.READING_SIZE:
                            toRead = Math.min(data.length - readHead, _this.readSizeRemaining);
                            data.copy(_this.readPartialSize, 4 - _this.readSizeRemaining, readHead, readHead + toRead);
                            _this.readSizeRemaining -= toRead;
                            readHead += toRead;
                            // readSizeRemaining >=0 here
                            if (_this.readSizeRemaining === 0) {
                                _this.readMessageSize = _this.readPartialSize.readUInt32BE(0);
                                _this.readMessageRemaining = _this.readMessageSize;
                                if (_this.readMessageRemaining > 0) {
                                    _this.readState = ReadState.READING_MESSAGE;
                                }
                                else {
                                    canPush = _this.tryPush(emptyBuffer, canPush);
                                    _this.readState = ReadState.NO_DATA;
                                }
                            }
                            break;
                        case ReadState.READING_MESSAGE:
                            toRead =
                                Math.min(data.length - readHead, _this.readMessageRemaining);
                            _this.readPartialMessage.push(data.slice(readHead, readHead + toRead));
                            _this.readMessageRemaining -= toRead;
                            readHead += toRead;
                            // readMessageRemaining >=0 here
                            if (_this.readMessageRemaining === 0) {
                                // At this point, we have read a full message
                                var messageBytes = Buffer.concat(_this.readPartialMessage, _this.readMessageSize);
                                // TODO(murgatroid99): Add receive message filters
                                canPush = _this.tryPush(messageBytes, canPush);
                                _this.readState = ReadState.NO_DATA;
                            }
                    }
                }
            });
            stream.on('end', function () {
                if (_this.unpushedReadMessages.length === 0) {
                    _this.push(null);
                }
                else {
                    _this.unpushedReadMessages.push(null);
                }
            });
            stream.on('streamClosed', function (errorCode) {
                var code;
                var details = '';
                switch (errorCode) {
                    case http2.constants.NGHTTP2_REFUSED_STREAM:
                        code = constants_1.Status.UNAVAILABLE;
                        break;
                    case http2.constants.NGHTTP2_CANCEL:
                        code = constants_1.Status.CANCELLED;
                        break;
                    case http2.constants.NGHTTP2_ENHANCE_YOUR_CALM:
                        code = constants_1.Status.RESOURCE_EXHAUSTED;
                        details = 'Bandwidth exhausted';
                        break;
                    case http2.constants.NGHTTP2_INADEQUATE_SECURITY:
                        code = constants_1.Status.PERMISSION_DENIED;
                        details = 'Protocol not secure enough';
                        break;
                    default:
                        code = constants_1.Status.INTERNAL;
                }
                _this.endCall({ code: code, details: details, metadata: new metadata_1.Metadata() });
            });
            stream.on('error', function () {
                _this.endCall({
                    code: constants_1.Status.INTERNAL,
                    details: 'Internal HTTP2 error',
                    metadata: new metadata_1.Metadata()
                });
            });
            if (!this.pendingRead) {
                stream.pause();
            }
            if (this.pendingWrite) {
                if (!this.pendingWriteCallback) {
                    throw new Error('Invalid state in write handling code');
                }
                stream.write(this.pendingWrite, this.pendingWriteCallback);
            }
            if (this.pendingFinalCallback) {
                stream.end(this.pendingFinalCallback);
            }
        }
    };
    Http2CallStream.prototype.cancelWithStatus = function (status, details) {
        this.endCall({ code: status, details: details, metadata: new metadata_1.Metadata() });
        if (this.http2Stream !== null) {
            /* TODO(murgatroid99): Determine if we want to send different RST_STREAM
             * codes based on the status code */
            this.http2Stream.rstWithCancel();
        }
    };
    Http2CallStream.prototype.getDeadline = function () {
        return this.options.deadline;
    };
    Http2CallStream.prototype.getCredentials = function () {
        return this.options.credentials;
    };
    Http2CallStream.prototype.getStatus = function () {
        return this.finalStatus;
    };
    Http2CallStream.prototype.getPeer = function () {
        throw new Error('Not yet implemented');
    };
    Http2CallStream.prototype._read = function (size) {
        if (this.http2Stream === null) {
            this.pendingRead = true;
        }
        else {
            while (this.unpushedReadMessages.length > 0) {
                var nextMessage = this.unpushedReadMessages.shift();
                var keepPushing = this.push(nextMessage);
                if (nextMessage === null || (!keepPushing)) {
                    return;
                }
            }
            /* Only resume reading from the http2Stream if we don't have any pending
             * messages to emit, and we haven't gotten the signal to stop pushing
             * messages */
            this.http2Stream.resume();
        }
    };
    // Encode a message to the wire format
    Http2CallStream.prototype.encodeMessage = function (message) {
        /* allocUnsafe doesn't initiate the bytes in the buffer. We are explicitly
         * overwriting every single byte, so that should be fine */
        var output = Buffer.allocUnsafe(message.message.length + 5);
        // TODO(murgatroid99): handle compressed flag appropriately
        output.writeUInt8(0, 0);
        output.writeUInt32BE(message.message.length, 1);
        message.message.copy(output, 5);
        return output;
    };
    Http2CallStream.prototype._write = function (chunk, encoding, cb) {
        // TODO(murgatroid99): Add send message filters
        var encodedMessage = this.encodeMessage(chunk);
        if (this.http2Stream === null) {
            this.pendingWrite = encodedMessage;
            this.pendingWriteCallback = cb;
        }
        else {
            this.http2Stream.write(encodedMessage, cb);
        }
    };
    Http2CallStream.prototype._final = function (cb) {
        if (this.http2Stream === null) {
            this.pendingFinalCallback = cb;
        }
        else {
            this.http2Stream.end(cb);
        }
    };
    return Http2CallStream;
}(stream_1.Duplex));
exports.Http2CallStream = Http2CallStream;

//# sourceMappingURL=call-stream.js.map
