"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var stream_1 = require("stream");
var constants_1 = require("./constants");
var ServiceErrorImpl = /** @class */ (function (_super) {
    __extends(ServiceErrorImpl, _super);
    function ServiceErrorImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return ServiceErrorImpl;
}(Error));
exports.ServiceErrorImpl = ServiceErrorImpl;
var ClientUnaryCallImpl = /** @class */ (function (_super) {
    __extends(ClientUnaryCallImpl, _super);
    function ClientUnaryCallImpl(call) {
        var _this = _super.call(this) || this;
        _this.call = call;
        call.on('metadata', function (metadata) {
            _this.emit('metadata', metadata);
        });
        return _this;
    }
    ClientUnaryCallImpl.prototype.cancel = function () {
        this.call.cancelWithStatus(constants_1.Status.CANCELLED, 'Cancelled on client');
    };
    ClientUnaryCallImpl.prototype.getPeer = function () {
        return this.call.getPeer();
    };
    return ClientUnaryCallImpl;
}(events_1.EventEmitter));
exports.ClientUnaryCallImpl = ClientUnaryCallImpl;
function setUpReadableStream(stream, call, deserialize) {
    call.on('data', function (data) {
        var deserialized;
        try {
            deserialized = deserialize(data);
        }
        catch (e) {
            call.cancelWithStatus(constants_1.Status.INTERNAL, 'Failed to parse server response');
            return;
        }
        if (!stream.push(deserialized)) {
            call.pause();
        }
    });
    call.on('end', function () {
        stream.push(null);
    });
    call.on('status', function (status) {
        stream.emit('status', status);
        if (status.code !== constants_1.Status.OK) {
            var error = new ServiceErrorImpl(status.details);
            error.code = status.code;
            error.metadata = status.metadata;
            stream.emit('error', error);
        }
    });
    call.pause();
}
var ClientReadableStreamImpl = /** @class */ (function (_super) {
    __extends(ClientReadableStreamImpl, _super);
    function ClientReadableStreamImpl(call, deserialize) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.call = call;
        _this.deserialize = deserialize;
        call.on('metadata', function (metadata) {
            _this.emit('metadata', metadata);
        });
        setUpReadableStream(_this, call, deserialize);
        return _this;
    }
    ClientReadableStreamImpl.prototype.cancel = function () {
        this.call.cancelWithStatus(constants_1.Status.CANCELLED, 'Cancelled on client');
    };
    ClientReadableStreamImpl.prototype.getPeer = function () {
        return this.call.getPeer();
    };
    ClientReadableStreamImpl.prototype._read = function (_size) {
        this.call.resume();
    };
    return ClientReadableStreamImpl;
}(stream_1.Readable));
exports.ClientReadableStreamImpl = ClientReadableStreamImpl;
function tryWrite(call, serialize, chunk, encoding, cb) {
    var message;
    var flags = Number(encoding);
    try {
        message = serialize(chunk);
    }
    catch (e) {
        call.cancelWithStatus(constants_1.Status.INTERNAL, 'Serialization failure');
        cb(e);
        return;
    }
    var writeObj = { message: message };
    if (!Number.isNaN(flags)) {
        writeObj.flags = flags;
    }
    call.write(writeObj, cb);
}
var ClientWritableStreamImpl = /** @class */ (function (_super) {
    __extends(ClientWritableStreamImpl, _super);
    function ClientWritableStreamImpl(call, serialize) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.call = call;
        _this.serialize = serialize;
        call.on('metadata', function (metadata) {
            _this.emit('metadata', metadata);
        });
        return _this;
    }
    ClientWritableStreamImpl.prototype.cancel = function () {
        this.call.cancelWithStatus(constants_1.Status.CANCELLED, 'Cancelled on client');
    };
    ClientWritableStreamImpl.prototype.getPeer = function () {
        return this.call.getPeer();
    };
    ClientWritableStreamImpl.prototype._write = function (chunk, encoding, cb) {
        tryWrite(this.call, this.serialize, chunk, encoding, cb);
    };
    ClientWritableStreamImpl.prototype._final = function (cb) {
        this.call.end();
        cb();
    };
    return ClientWritableStreamImpl;
}(stream_1.Writable));
exports.ClientWritableStreamImpl = ClientWritableStreamImpl;
var ClientDuplexStreamImpl = /** @class */ (function (_super) {
    __extends(ClientDuplexStreamImpl, _super);
    function ClientDuplexStreamImpl(call, serialize, deserialize) {
        var _this = _super.call(this, { objectMode: true }) || this;
        _this.call = call;
        _this.serialize = serialize;
        _this.deserialize = deserialize;
        call.on('metadata', function (metadata) {
            _this.emit('metadata', metadata);
        });
        setUpReadableStream(_this, call, deserialize);
        return _this;
    }
    ClientDuplexStreamImpl.prototype.cancel = function () {
        this.call.cancelWithStatus(constants_1.Status.CANCELLED, 'Cancelled on client');
    };
    ClientDuplexStreamImpl.prototype.getPeer = function () {
        return this.call.getPeer();
    };
    ClientDuplexStreamImpl.prototype._read = function (_size) {
        this.call.resume();
    };
    ClientDuplexStreamImpl.prototype._write = function (chunk, encoding, cb) {
        tryWrite(this.call, this.serialize, chunk, encoding, cb);
    };
    ClientDuplexStreamImpl.prototype._final = function (cb) {
        this.call.end();
        cb();
    };
    return ClientDuplexStreamImpl;
}(stream_1.Duplex));
exports.ClientDuplexStreamImpl = ClientDuplexStreamImpl;

//# sourceMappingURL=call.js.map
