"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var tls_1 = require("tls");
var call_credentials_1 = require("./call-credentials");
var ChannelCredentialsImpl = /** @class */ (function () {
    function ChannelCredentialsImpl(callCredentials) {
        this.callCredentials = callCredentials || call_credentials_1.CallCredentials.createEmpty();
    }
    ChannelCredentialsImpl.prototype.getCallCredentials = function () {
        return this.callCredentials;
    };
    return ChannelCredentialsImpl;
}());
var InsecureChannelCredentialsImpl = /** @class */ (function (_super) {
    __extends(InsecureChannelCredentialsImpl, _super);
    function InsecureChannelCredentialsImpl(callCredentials) {
        return _super.call(this, callCredentials) || this;
    }
    InsecureChannelCredentialsImpl.prototype.compose = function (callCredentials) {
        throw new Error('Cannot compose insecure credentials');
    };
    InsecureChannelCredentialsImpl.prototype.getSecureContext = function () {
        return null;
    };
    return InsecureChannelCredentialsImpl;
}(ChannelCredentialsImpl));
var SecureChannelCredentialsImpl = /** @class */ (function (_super) {
    __extends(SecureChannelCredentialsImpl, _super);
    function SecureChannelCredentialsImpl(secureContext, callCredentials) {
        var _this = _super.call(this, callCredentials) || this;
        _this.secureContext = secureContext;
        return _this;
    }
    SecureChannelCredentialsImpl.prototype.compose = function (callCredentials) {
        var combinedCallCredentials = this.callCredentials.compose(callCredentials);
        return new SecureChannelCredentialsImpl(this.secureContext, combinedCallCredentials);
    };
    SecureChannelCredentialsImpl.prototype.getSecureContext = function () {
        return this.secureContext;
    };
    return SecureChannelCredentialsImpl;
}(ChannelCredentialsImpl));
var ChannelCredentials;
(function (ChannelCredentials) {
    /**
     * Return a new ChannelCredentials instance with a given set of credentials.
     * The resulting instance can be used to construct a Channel that communicates
     * over TLS.
     * @param rootCerts The root certificate data.
     * @param privateKey The client certificate private key, if available.
     * @param certChain The client certificate key chain, if available.
     */
    function createSsl(rootCerts, privateKey, certChain) {
        if (privateKey && !certChain) {
            throw new Error('Private key must be given with accompanying certificate chain');
        }
        if (!privateKey && certChain) {
            throw new Error('Certificate chain must be given with accompanying private key');
        }
        var secureContext = tls_1.createSecureContext({
            ca: rootCerts || undefined,
            key: privateKey || undefined,
            cert: certChain || undefined
        });
        return new SecureChannelCredentialsImpl(secureContext);
    }
    ChannelCredentials.createSsl = createSsl;
    /**
     * Return a new ChannelCredentials instance with no credentials.
     */
    function createInsecure() {
        return new InsecureChannelCredentialsImpl();
    }
    ChannelCredentials.createInsecure = createInsecure;
})(ChannelCredentials = exports.ChannelCredentials || (exports.ChannelCredentials = {}));

//# sourceMappingURL=channel-credentials.js.map
