"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var http2 = require("http2");
var call_credentials_1 = require("./call-credentials");
var call_credentials_filter_1 = require("./call-credentials-filter");
var call_stream_1 = require("./call-stream");
var compression_filter_1 = require("./compression-filter");
var constants_1 = require("./constants");
var deadline_filter_1 = require("./deadline-filter");
var filter_stack_1 = require("./filter-stack");
var IDLE_TIMEOUT_MS = 300000;
var _a = http2.constants, HTTP2_HEADER_AUTHORITY = _a.HTTP2_HEADER_AUTHORITY, HTTP2_HEADER_CONTENT_TYPE = _a.HTTP2_HEADER_CONTENT_TYPE, HTTP2_HEADER_METHOD = _a.HTTP2_HEADER_METHOD, HTTP2_HEADER_PATH = _a.HTTP2_HEADER_PATH, HTTP2_HEADER_SCHEME = _a.HTTP2_HEADER_SCHEME, HTTP2_HEADER_TE = _a.HTTP2_HEADER_TE;
var ConnectivityState;
(function (ConnectivityState) {
    ConnectivityState[ConnectivityState["CONNECTING"] = 0] = "CONNECTING";
    ConnectivityState[ConnectivityState["READY"] = 1] = "READY";
    ConnectivityState[ConnectivityState["TRANSIENT_FAILURE"] = 2] = "TRANSIENT_FAILURE";
    ConnectivityState[ConnectivityState["IDLE"] = 3] = "IDLE";
    ConnectivityState[ConnectivityState["SHUTDOWN"] = 4] = "SHUTDOWN";
})(ConnectivityState = exports.ConnectivityState || (exports.ConnectivityState = {}));
var Http2Channel = /** @class */ (function (_super) {
    __extends(Http2Channel, _super);
    function Http2Channel(address, credentials, options) {
        var _this = _super.call(this) || this;
        _this.address = address;
        _this.credentials = credentials;
        _this.options = options;
        _this.connectivityState = ConnectivityState.IDLE;
        _this.idleTimerId = null;
        if (credentials.getSecureContext() === null) {
            address.protocol = 'http';
        }
        else {
            address.protocol = 'https';
        }
        _this.filterStackFactory = new filter_stack_1.FilterStackFactory([
            new compression_filter_1.CompressionFilterFactory(_this),
            new call_credentials_filter_1.CallCredentialsFilterFactory(_this), new deadline_filter_1.DeadlineFilterFactory(_this)
        ]);
        return _this;
    }
    Http2Channel.prototype.transitionToState = function (newState) {
        if (newState !== this.connectivityState) {
            this.connectivityState = newState;
            this.emit('connectivityStateChanged', newState);
        }
    };
    Http2Channel.prototype.startConnecting = function () {
        var _this = this;
        this.transitionToState(ConnectivityState.CONNECTING);
        var secureContext = this.credentials.getSecureContext();
        if (secureContext === null) {
            this.subChannel = http2.connect(this.address);
        }
        else {
            this.subChannel = http2.connect(this.address, { secureContext: secureContext });
        }
        this.subChannel.once('connect', function () {
            _this.transitionToState(ConnectivityState.READY);
        });
        this.subChannel.setTimeout(IDLE_TIMEOUT_MS, function () {
            _this.goIdle();
        });
        /* TODO(murgatroid99): add connection-level error handling with exponential
         * reconnection backoff */
    };
    Http2Channel.prototype.goIdle = function () {
        if (this.subChannel !== null) {
            this.subChannel.shutdown({ graceful: true }, function () { return undefined; });
            this.subChannel = null;
        }
        this.transitionToState(ConnectivityState.IDLE);
    };
    Http2Channel.prototype.kickConnectivityState = function () {
        if (this.connectivityState === ConnectivityState.IDLE) {
            this.startConnecting();
        }
    };
    Http2Channel.prototype.startHttp2Stream = function (methodName, stream, metadata) {
        var _this = this;
        var finalMetadata = stream.filterStack.sendMetadata(Promise.resolve(metadata));
        this.connect(function () {
            finalMetadata.then(function (metadataValue) {
                var headers = metadataValue.toHttp2Headers();
                headers[HTTP2_HEADER_AUTHORITY] = _this.address.hostname;
                headers[HTTP2_HEADER_CONTENT_TYPE] = 'application/grpc';
                headers[HTTP2_HEADER_METHOD] = 'POST';
                headers[HTTP2_HEADER_PATH] = methodName;
                headers[HTTP2_HEADER_TE] = 'trailers';
                if (stream.getStatus() === null) {
                    if (_this.connectivityState === ConnectivityState.READY) {
                        var session = _this.subChannel;
                        stream.attachHttp2Stream(session.request(headers));
                    }
                    else {
                        /* In this case, we lost the connection while finalizing
                         * metadata. That should be very unusual */
                        setImmediate(function () {
                            _this.startHttp2Stream(methodName, stream, metadata);
                        });
                    }
                }
            }, function (error) {
                stream.cancelWithStatus(constants_1.Status.UNKNOWN, 'Failed to generate metadata');
            });
        });
    };
    Http2Channel.prototype.createStream = function (methodName, metadata, options) {
        if (this.connectivityState === ConnectivityState.SHUTDOWN) {
            throw new Error('Channel has been shut down');
        }
        var finalOptions = {
            deadline: options.deadline === undefined ? Infinity : options.deadline,
            credentials: options.credentials || call_credentials_1.CallCredentials.createEmpty(),
            flags: options.flags || 0
        };
        var stream = new call_stream_1.Http2CallStream(methodName, finalOptions, this.filterStackFactory);
        this.startHttp2Stream(methodName, stream, metadata);
        return stream;
    };
    Http2Channel.prototype.connect = function (callback) {
        this.kickConnectivityState();
        if (this.connectivityState === ConnectivityState.READY) {
            setImmediate(callback);
        }
        else {
            this.once('connectivityStateChanged', function (newState) {
                if (newState === ConnectivityState.READY) {
                    callback();
                }
            });
        }
    };
    Http2Channel.prototype.getConnectivityState = function () {
        return this.connectivityState;
    };
    Http2Channel.prototype.close = function () {
        if (this.connectivityState === ConnectivityState.SHUTDOWN) {
            throw new Error('Channel has been shut down');
        }
        this.transitionToState(ConnectivityState.SHUTDOWN);
        if (this.subChannel !== null) {
            this.subChannel.shutdown({ graceful: true });
        }
    };
    return Http2Channel;
}(events_1.EventEmitter));
exports.Http2Channel = Http2Channel;

//# sourceMappingURL=channel.js.map
