"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
var FilterStack = /** @class */ (function () {
    function FilterStack(filters) {
        this.filters = filters;
    }
    FilterStack.prototype.sendMetadata = function (metadata) {
        return lodash_1.flow(lodash_1.map(this.filters, function (filter) { return filter.sendMetadata.bind(filter); }))(metadata);
    };
    FilterStack.prototype.receiveMetadata = function (metadata) {
        return lodash_1.flowRight(lodash_1.map(this.filters, function (filter) { return filter.receiveMetadata.bind(filter); }))(metadata);
    };
    FilterStack.prototype.receiveTrailers = function (status) {
        return lodash_1.flowRight(lodash_1.map(this.filters, function (filter) { return filter.receiveTrailers.bind(filter); }))(status);
    };
    return FilterStack;
}());
exports.FilterStack = FilterStack;
var FilterStackFactory = /** @class */ (function () {
    function FilterStackFactory(factories) {
        this.factories = factories;
    }
    FilterStackFactory.prototype.createFilter = function (callStream) {
        return new FilterStack(lodash_1.map(this.factories, function (factory) { return factory.createFilter(callStream); }));
    };
    return FilterStackFactory;
}());
exports.FilterStackFactory = FilterStackFactory;

//# sourceMappingURL=filter-stack.js.map
